searchNodes=[{"doc":"Bandit is an HTTP server for Plug and WebSock apps. As an HTTP server, Bandit's primary goal is to act as 'glue' between client connections managed by Thousand Island and application code defined via the Plug and/or WebSock APIs. As such there really isn't a whole lot of user-visible surface area to Bandit, and as a consequence the API documentation presented here is somewhat sparse. This is by design! Bandit is intended to 'just work' in almost all cases; the only thought users typically have to put into Bandit comes in the choice of which options (if any) they would like to change when starting a Bandit server. The sparseness of the Bandit API should not be taken as an indicator of the comprehensiveness or robustness of the project. Using Bandit With Phoenix Bandit fully supports Phoenix. Phoenix applications which use WebSockets for features such as Channels or LiveView require Phoenix 1.7 or later. Using Bandit to host your Phoenix application couldn't be simpler: Add Bandit as a dependency in your Phoenix application's mix.exs : { :bandit , &quot;&gt;= 0.5.10&quot; } Add the following to your endpoint configuration in config/config.exs : config :your_app , YourAppWeb.Endpoint , adapter : Bandit.PhoenixAdapter That's it! You should now see messages at startup indicating that Phoenix is using Bandit to serve your endpoint. For more details about how to configure Bandit within Phoenix, consult the Bandit.PhoenixAdapter documentation. Using Bandit With Plug Applications Using Bandit to host your own Plug is very straightforward. Assuming you have a Plug module implemented already, you can host it within Bandit by adding something similar to the following to your application's Application.start/2 function: def start ( _type , _args ) do children = [ { Bandit , plug : MyApp.MyPlug , scheme : :http , options : [ port : 4000 ] } ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end For details about writing Plug based applications, consult the excellent Plug documentation for plenty of examples &amp; tips to get started. Bandit supports the complete Plug API &amp; should work correctly with any Plug-based application. If you encounter errors using Bandit your Plug app, please do get in touch by filing an issue on the Bandit GitHub project (especially if the error does not occur with another HTTP server such as Cowboy). Config Options Bandit takes a number of options at startup: plug : The plug to handle connections. Can be specified as MyPlug or {MyPlug, plug_opts} display_plug : The plug to use when describing the connection in logs. Useful for situations such as Phoenix code reloading where you have a 'wrapper' plug but wish to refer to the connection by the endpoint name scheme : One of :http or :https . If :https is specified, you will need to specify certfile and keyfile in the transport_options subsection of options . options : Options to pass to ThousandIsland . For an exhaustive list of options see the ThousandIsland documentation, however some common options are: port : The port to bind to. Defaults to 4000 num_acceptors : The number of acceptor processes to run. This is mostly a performance tuning knob and can usually be left at the default value of 10 read_timeout : How long to wait for data from the client before timing out and closing the connection, specified in milliseconds. Defaults to 60_000 milliseconds shutdown_timeout : How long to wait for existing connections to complete before forcibly shutting them down at server shutdown, specified in milliseconds. Defaults to 15_000 milliseconds. May also be :infinity or :brutal_kill as described in the Supervisor documentation. transport_module : The name of the module which provides basic socket functions. This overrides any value set for scheme and is intended for cases where control over the socket at a fundamental level is needed. transport_options : A keyword list of options to be passed into the transport socket's listen function http_1_options : Options to configure the HTTP/1 stack in Bandit. Valid options are: enabled : Whether or not to serve HTTP/1 requests. Defaults to true max_request_line_length : The maximum permitted length of the request line (expressed as the number of bytes on the wire) in an HTTP/1.1 request. Defaults to 10_000 bytes max_header_length : The maximum permitted length of any single header (combined key &amp; value, expressed as the number of bytes on the wire) in an HTTP/1.1 request. Defaults to 10_000 bytes max_header_count : The maximum permitted number of headers in an HTTP/1.1 request. Defaults to 50 headers max_requests : The maximum number of requests to serve in a single HTTP/1.1 connection before closing the connection. Defaults to 0 (no limit) http_2_options : Options to configure the HTTP/2 stack in Bandit. Valid options are: enabled : Whether or not to serve HTTP/2 requests. Defaults to true max_header_key_length : The maximum permitted length of any single header key (expressed as the number of decompressed bytes) in an HTTP/2 request. Defaults to 10_000 bytes max_header_value_length : The maximum permitted length of any single header value (expressed as the number of decompressed bytes) in an HTTP/2 request. Defaults to 10_000 bytes max_header_count : The maximum permitted number of headers in an HTTP/2 request. Defaults to 50 headers max_requests : The maximum number of requests to serve in a single HTTP/2 connection before closing the connection. Defaults to 0 (no limit) default_local_settings : Options to override the default values for local HTTP/2 settings. Values provided here will override the defaults specified in RFC7540§6.5.2. websocket_options : Options to configure the WebSocket stack in Bandit. Valid options are: enabled : Whether or not to serve WebSocket upgrade requests. Defaults to true max_frame_size : The maximum size of a single WebSocket frame (expressed as a number of bytes on the wire). Defaults to 0 (no limit) validate_text_frames : Whether or not to validate text frames as being UTF-8. Strictly speaking this is required per RFC6455§5.6, however it can be an expensive operation and one that may be safely skipped in some situations. Defaults to true compress : Whether or not to allow per-message deflate compression globally. Note that upgrade requests still need to set the compress: true option in connection_opts on a per-upgrade basis for compression to be negotiated (see 'WebSocket Support' section below for details). Defaults to true deflate_opts : A keyword list of options to set on the deflate library. Possible options are: level : The compression level to use for deflation. May be one of none , default , best_compression , best_speed , or an integer in 0..9 . See :zlib documentation for more information. Defaults to default memory_level : The memory level to use for deflation. May be an integer in 1..9 . See :zlib documentation for more information. Defaults to 8 strategy : The strategy to use for deflation. May be one of default , filtered , huffman_only , or rle . See :zlib documentation for more information. Defaults to default Setting up an HTTPS Server By far the most common stumbling block encountered with configuration involves setting up an HTTPS server. Bandit is comparatively easy to set up in this regard, with a working example looking similar to the following: def start ( _type , _args ) do bandit_options = [ port : 4000 , transport_options : [ certfile : Path . join ( __DIR__ , &quot;path/to/cert.pem&quot; ) , keyfile : Path . join ( __DIR__ , &quot;path/to/key.pem&quot; ) ] ] children = [ { Bandit , plug : MyApp.MyPlug , scheme : :https , options : bandit_options } ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end WebSocket Support Bandit supports upgrading HTTP requests to WebSocket connections via the use of the Plug.Conn.upgrade_adapter/3 function, called with :websocket as the second argument. Applications should validate that the connection represents a valid WebSocket request before calling this function (Bandit will validate the connection as part of the upgrade process, but does not provide any capacity for an application to be notified if the upgrade is not successful). If an application wishes to negotiate WebSocket subprotocols or otherwise set any response headers, it should do so before calling Plug.Conn.upgrade_adapter/3 . The third argument to Plug.Conn.upgrade_adapter/3 defines the details of how Bandit should handle the WebSocket connection, and must take the form {handler, handler_opts, connection_opts} , where values are as follows: handler is a module which implements the WebSock API handler_opts is an arbitrary term which will be passed as the argument to WebSock.init/1 connection_opts is a keyword list which consists of zero or more of the following options: timeout : The number of milliseconds to wait after no client data is received before closing the connection. Defaults to 60_000 compress : Whether or not to accept negotiation of a compression extension with the client. Defaults to false fullsweep_after : The maximum number of garbage collections before forcing a full sweep of the socket process. You can set it to 0 to force more frequent cleanups of your websocket transport processes. Setting this option requires Erlang/OTP 24","ref":"Bandit.html","title":"Bandit","type":"module"},{"doc":"","ref":"Bandit.html#child_spec/1","title":"Bandit.child_spec/1","type":"function"},{"doc":"Starts a Bandit server using the provided arguments. See &quot;Config Options' above for specific options to pass to this function.","ref":"Bandit.html#start_link/1","title":"Bandit.start_link/1","type":"function"},{"doc":"A Plug definition","ref":"Bandit.html#t:plug/0","title":"Bandit.plug/0","type":"type"},{"doc":"Logging conveniences for Bandit servers Allows dynamically adding and altering the log level used to trace connections within a Bandit server via the use of telemetry hooks. Should you wish to do your own logging or tracking of these events, a complete list of the telemetry events emitted by Bandit is described in the module documentation for Bandit.Telemetry . The logging included in this module is concerned specifically with protocol level events. Should you wish to log lower level transport concens, there are similar functions to these in the ThousandIsland.Logger module. Corresponding telemetry events are described in the module documentation for ThousandIsland.Telemetry .","ref":"Bandit.Logger.html","title":"Bandit.Logger","type":"module"},{"doc":"Start logging Bandit at the specified log level. Valid values for log level are :error and :info . Enabling a given log level implicitly enables all higher log levels as well.","ref":"Bandit.Logger.html#attach_logger/1","title":"Bandit.Logger.attach_logger/1","type":"function"},{"doc":"Stop logging Thousand Island at the specified log level. Disabling a given log level implicitly disables all lower log levels as well.","ref":"Bandit.Logger.html#detach_logger/1","title":"Bandit.Logger.detach_logger/1","type":"function"},{"doc":"Supported log levels","ref":"Bandit.Logger.html#t:log_level/0","title":"Bandit.Logger.log_level/0","type":"type"},{"doc":"A Bandit adapter for Phoenix. WebSocket support requires a version of Phoenix with Plug upgrade support, which is available as part of Phoenix 1.7 and later. This module will work fine on earlier versions of Phoenix, just without WebSocket support. To use this adapter, your project will need to include Bandit as a dependency; see https://hex.pm/bandit for details on the currently supported version of Bandit to include. Once Bandit is included as a dependency of your Phoenix project, add the following to your endpoint configuration in config/config.exs : config :your_app , YourAppWeb.Endpoint , adapter : Bandit.PhoenixAdapter Endpoint configuration Configuring Bandit within your Phoenix app is done in largely the same way as configuration for Cowboy works. For the most part, your existing configuration within your /config/*.exs files will work unchanged, although some of the more exotic options are different. Bandit supports the following parameters within the :http and :https parameters: :http : the configuration for the HTTP server. Accepts the following options: port : The port to run on. Defaults to 4000. Note that if a Unix domain socket is specified in the ip option, the value of port must be 0 . ip : The address to bind to. Can be specified as a 4-element tuple such as {127, 0, 0, 1} for IPv4 addresses, an 8-element tuple for IPv6 addresses, or using {:local, path} to bind to a Unix domain socket. Defaults to the Bandit default of {0, 0, 0, 0, 0, 0, 0, 0} . transport_options : Any valid value from ThousandIsland.Transports.TCP Defaults to false , which will cause Bandit to not start an HTTP server. :https : the configuration for the HTTPS server. Accepts the following options: port : The port to run on. Defaults to 4040. Note that if a Unix domain socket is specified in the ip option, the value of port must be 0 . ip : The address to bind to. Can be specified as a 4-element tuple such as {127, 0, 0, 1} for IPv4 addresses, an 8-element tuple for IPv6 addresses, or using {:local, path} to bind to a Unix domain socket. Defaults to the Bandit default of {0, 0, 0, 0, 0, 0, 0, 0} . transport_options : Any valid value from ThousandIsland.Transports.SSL Defaults to false , which will cause Bandit to not start an HTTPS server.","ref":"Bandit.PhoenixAdapter.html","title":"Bandit.PhoenixAdapter","type":"module"},{"doc":"The following telemetry spans are emitted by bandit [:bandit, :request, *] Represents Bandit handling a specific client HTTP request This span is started by the following event: [:bandit, :request, :start] Represents the start of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units This event contains the following metadata: telemetry_span_context : A unique identifier for this span connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request This span is ended by the following event: [:bandit, :request, :stop] Represents the end of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units duration : The span duration, in :native units conn : The Plug.Conn representing this connection req_header_end_time : The time that header reading completed, in :native units req_body_start_time : The time that request body reading started, in :native units. req_body_end_time : The time that request body reading completed, in :native units req_line_bytes : The length of the request line, in octets. Includes all line breaks. Not included for HTTP/2 requests req_header_bytes : The length of the request headers, in octets. Includes all line breaks. Not included for HTTP/2 requests req_body_bytes : The length of the request body, in octets resp_start_time : The time that the response started, in :native units resp_end_time : The time that the response completed, in :native units. Not included for chunked responses resp_line_bytes : The length of the reponse line, in octets. Includes all line breaks. Not included for HTTP/2 requests resp_header_bytes : The length of the reponse headers, in octets. Includes all line breaks. Not included for HTTP/2 requests resp_body_bytes : The length of the reponse body, in octets. Set to 0 for chunked responses This event contains the following metadata: telemetry_span_context : A unique identifier for this span connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request error : The error that caused the span to end, if it ended in error The following events may be emitted within this span: [:bandit, :request, :exception] The request for this span ended unexpectedly This event contains the following measurements: monotonic_time : The time of this event, in :native units This event contains the following metadata: telemetry_span_context : A unique identifier for this span connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request kind : The kind of unexpected condition, typically :exit exception : The exception which caused this unexpected termination stacktrace : The stacktrace of the location which caused this unexpected termination [:bandit, :websocket, *] Represents Bandit handling a WebSocket connection This span is started by the following event: [:bandit, :websocket, :start] Represents the start of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units compress : Details about the compression configuration for this connection This event contains the following metadata: telemetry_span_context : A unique identifier for this span origin_telemetry_span_context : The span context of the Bandit :request span from which this connection originated connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request This span is ended by the following event: [:bandit, :websocket, :stop] Represents the end of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units duration : The span duration, in :native units recv_text_frame_count : The number of text frames received recv_text_frame_bytes : The total number of bytes received in the payload of text frames recv_binary_frame_count : The number of binary frames received recv_binary_frame_bytes : The total number of bytes received in the payload of binary frames recv_ping_frame_count : The number of ping frames received recv_ping_frame_bytes : The total number of bytes received in the payload of ping frames recv_pong_frame_count : The number of pong frames received recv_pong_frame_bytes : The total number of bytes received in the payload of pong frames recv_connection_close_frame_count : The number of connection close frames received recv_connection_close_frame_bytes : The total number of bytes received in the payload of connection close frames recv_continuation_frame_count : The number of continuation frames received recv_continuation_frame_bytes : The total number of bytes received in the payload of continuation frames send_text_frame_count : The number of text frames sent send_text_frame_bytes : The total number of bytes sent in the payload of text frames send_binary_frame_count : The number of binary frames sent send_binary_frame_bytes : The total number of bytes sent in the payload of binary frames send_ping_frame_count : The number of ping frames sent send_ping_frame_bytes : The total number of bytes sent in the payload of ping frames send_pong_frame_count : The number of pong frames sent send_pong_frame_bytes : The total number of bytes sent in the payload of pong frames send_connection_close_frame_count : The number of connection close frames sent send_connection_close_frame_bytes : The total number of bytes sent in the payload of connection close frames send_continuation_frame_count : The number of continuation frames sent send_continuation_frame_bytes : The total number of bytes sent in the payload of continuation frames This event contains the following metadata: telemetry_span_context : A unique identifier for this span origin_telemetry_span_context : The span context of the Bandit :request span from which this connection originated connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request error : The error that caused the span to end, if it ended in error","ref":"Bandit.Telemetry.html","title":"Bandit.Telemetry","type":"module"},{"doc":"See System.monotonic_time/0 .","ref":"Bandit.Telemetry.html#monotonic_time/0","title":"Bandit.Telemetry.monotonic_time/0","type":"function"},{"doc":"","ref":"Bandit.Telemetry.html#span_exception/4","title":"Bandit.Telemetry.span_exception/4","type":"function"},{"doc":"","ref":"Bandit.Telemetry.html#t:t/0","title":"Bandit.Telemetry.t/0","type":"opaque"},{"doc":"Raised by Bandit adapters if a body is attempted to be read more than once per request","ref":"Bandit.BodyAlreadyReadError.html","title":"Bandit.BodyAlreadyReadError","type":"exception"},{"doc":"","ref":"Bandit.HTTP2.Stream.StreamError.html","title":"Bandit.HTTP2.Stream.StreamError","type":"exception"}]